@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-__version__ = "0.0.7"
+__version__ = "0.0.10"
 __desc__ = "Automate process of downloading name your price albums from bandcamp."
 
 import argparse
@@ -9,7 +9,9 @@ import os
 import re
 import shutil
 import sys
+from enum import IntEnum
 from pathlib import Path
+from urllib.parse import urljoin, urlparse
 
 import requests
 from selenium import webdriver
@@ -32,6 +34,13 @@ drivers = (
 )
 
 
+class ExitCodes(IntEnum):
+    SUCCESS = 0
+    UNDOWNLOADABLE = 1
+    EMAIL_UNSPECIFIED = 2
+    CACHE_CORRUPTED = 3
+
+
 def main():
     parser = argparse.ArgumentParser(
         prog="bandcamp_name_your_price_dl",
@@ -69,21 +78,21 @@ def main():
         "--skip-nyp-check",
         "--skip-name-your-price-check",
         action="store_true",
-        help="don't check if album is name your price before downloading",
+        help="don't check if album is name your price before trying to download",
     )
     parser.add_argument(
         "--wait-time",
         metavar="SECONDS",
         type=int,
         default=10,
-        help="period to wait for pages loading",
+        help="period to wait for pages loading (in seconds) (default is 10)",
     )
     parser.add_argument(
         "--preparing-wait-time",
         metavar="SECONDS",
         type=int,
         default=60,
-        help="period to wait for bandcamp preparing download",
+        help="period to wait for bandcamp preparing download (in seconds) (default is 60)",
     )
     parser.add_argument(
         "--driver",
@@ -111,7 +120,7 @@ def main():
     parser.add_argument(
         "--ignore-cache",
         action="store_true",
-        help="don't load cache and pretend it is empty",
+        help="don't load cache and don't write anything to it",
     )
     parser.add_argument(
         "--email",
@@ -133,7 +142,7 @@ def main():
     )
     args = parser.parse_args(sys.argv[1:])
 
-    album_url = args.album_url
+    album_url = remove_url_query_parameters(args.album_url)
     download_url = None
     local_file_name = None
     driver = None
@@ -169,25 +178,26 @@ def main():
         except json.JSONDecodeError as e:
             print(e.msg)
             if not ask_to_overwrite_cache():
-                exit(3)
+                exit(ExitCodes.CACHE_CORRUPTED)
     else:
         loaded_cache = []
 
     def write_cache():
         if not args.ignore_cache:
             cache_entry["album_url"] = album_url
-            loaded_cache.append(cache_entry)
+            if cache_entry not in loaded_cache:
+                loaded_cache.append(cache_entry)
             with open(cache_file, "w") as f:
                 json.dump(loaded_cache, f)
 
     # Search for entry with desired url in cache, if not found then create new one
 
     for entry in loaded_cache:
-        if entry["album_url"] == album_url:
+        if remove_url_query_parameters(entry["album_url"]) == album_url:
             try:
                 if "downloadable" in entry.keys() and not entry["downloadable"]:
                     eprint("Album marked as undownloadable in cache. Aborting.")
-                    exit(1)
+                    exit(ExitCodes.UNDOWNLOADABLE)
                 download_url = entry["download_url"]
                 local_file_name = entry["local_file_name"]
                 cache_entry = entry
@@ -196,7 +206,7 @@ def main():
             except KeyError as e:
                 print(e.msg)
                 if not ask_to_overwrite_cache():
-                    return 3
+                    exit(ExitCodes.CACHE_CORRUPTED)
     else:
         loaded_cache.append({"album_url": album_url})
         cache_entry = loaded_cache[-1]
@@ -232,9 +242,12 @@ def main():
             f"File exists in {file_name_to_print}. Skipping scrapping and downloading.",
             "Rerun program with --dont-skip-if-file-exists to redownload.",
         )
-        finish_and_exit(0)
+        finish_and_exit(ExitCodes.SUCCESS)
 
-    if download_url is None or requests.head(download_url).status_code != 200:
+    if (
+        download_url is None
+        or requests.get(download_url, stream=True).status_code != 200
+    ):
         if args.encoding is not None:
             if args.encoding == "mp3":
                 onsite_encoding = "MP3 320"
@@ -263,7 +276,7 @@ def main():
             profile.set_preference("permissions.default.image", 2)
             if not args.show_browser_window:
                 os.environ["MOZ_HEADLESS"] = "1"
-            driver = webdriver.Firefox(profile)
+            driver = webdriver.Firefox(profile, service_log_path=os.devnull)
         elif args.driver == "opera":
             driver = webdriver.Opera()
         elif args.driver == "phantomjs":
@@ -303,14 +316,14 @@ def main():
                     eprint("Album is not name your price. Aborting.")
                     cache_entry["downloadable"] = False
                     write_cache()
-                    finish_and_exit(1)
+                    finish_and_exit(ExitCodes.UNDOWNLOADABLE)
             except NoSuchElementException:
                 eprint(
                     "Element indicating if is album name your price not found. Aborting."
                 )
                 cache_entry["downloadable"] = False
                 write_cache()
-                finish_and_exit(1)
+                finish_and_exit(ExitCodes.UNDOWNLOADABLE)
 
             try:
                 buy_link = driver.find_element_by_xpath(
@@ -321,7 +334,7 @@ def main():
                 eprint('"Buy Digital Album" link not found. Aborting')
                 cache_entry["downloadable"] = False
                 write_cache()
-                finish_and_exit(1)
+                finish_and_exit(ExitCodes.UNDOWNLOADABLE)
 
             price_input_filled = driver.find_element_by_xpath(
                 "//input[@class='display-price numeric']"
@@ -351,7 +364,7 @@ def main():
                         "Bandcamp asked for email, but no email address or postal code specified."
                         " Aborting."
                     )
-                    finish_and_exit(2)
+                    finish_and_exit(ExitCodes.EMAIL_UNSPECIFIED)
                 if country_abbrev is not None:
                     country_dropdown_list = Select(
                         driver.find_element_by_xpath("//*[@id='fan_email_country']")
@@ -431,7 +444,11 @@ def main():
         cache_entry["local_file_name"] = local_file_name
         write_cache()
 
-    finish_and_exit(0)
+    finish_and_exit(ExitCodes.SUCCESS)
+
+
+def remove_url_query_parameters(url):
+    return urljoin(url, urlparse(url).path)
 
 
 def eprint(*args, **kwargs):
